package poi.tests;

import java.io.File ;
import java.io.FileInputStream ;
import java.io.FileNotFoundException ;
import java.io.IOException ;

import org.apache.poi.hssf.usermodel.HSSFFormulaEvaluator ;
import org.apache.poi.hssf.usermodel.HSSFWorkbook ;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException ;
import org.apache.poi.ss.formula.functions.FreeRefFunction ;
import org.apache.poi.ss.formula.udf.AggregatingUDFFinder ;
import org.apache.poi.ss.formula.udf.DefaultUDFFinder ;
import org.apache.poi.ss.formula.udf.UDFFinder ;
import org.apache.poi.ss.usermodel.Cell ;
import org.apache.poi.ss.usermodel.CellValue ;
import org.apache.poi.ss.usermodel.Row ;
import org.apache.poi.ss.usermodel.Sheet ;
import org.apache.poi.ss.usermodel.Workbook ;
import org.apache.poi.ss.usermodel.WorkbookFactory ;
import org.apache.poi.ss.util.CellReference ;


public class UserDefinedFunctionExample {

    
    
    public static void main( String[] args ) {
        
        if( args == null || ( args != null && args.length !=2 ) ) {
            System.out.println( "usage: java UserDefinedFunctionExample fileName cellId" ) ;
        }
        
        System.out.println( "fileName: " + args[0] ) ;
        System.out.println( "cell: " + args[1] ) ;
        
        File workbookFile = new File( args[0] ) ;
        
        try {
            FileInputStream fis = new FileInputStream(workbookFile);
            Workbook workbook = WorkbookFactory.create(fis);
            
            String[] names = { "calculatePayment" } ;
            FreeRefFunction[] functions = { new CalculateMortgage() } ; 
            
            UDFFinder udff1 = new DefaultUDFFinder( names, functions ) ;
            UDFFinder udff = new AggregatingUDFFinder( udff1 ) ;
            
            HSSFFormulaEvaluator evaluator = HSSFFormulaEvaluator.create(
                    (HSSFWorkbook) workbook, null, udff) ;

            
            CellReference cr = new CellReference( args[1] ) ;
            String sheetName = cr.getSheetName() ;
            Sheet sheet = workbook.getSheet( sheetName ) ;
            int rowIdx = cr.getRow() ;
            int colIdx = cr.getCol() ;
            Row row = sheet.getRow( rowIdx ) ;
            Cell cell = row.getCell( colIdx ) ;
            
            CellValue value = evaluator.evaluate( cell ) ;
            
            System.out.println("returns value: " +  value ) ;
            
            
        } catch( FileNotFoundException e ) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch( InvalidFormatException e ) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch( IOException e ) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
